{ SelFolderDlg.pas

  TSelectFolderDialog component (can be registered in Dialogs components panel)

  by Deefaze 01/22/2010
}
unit SelFolderDlg;

interface

uses
  Windows,
  SysUtils,
  FileCtrl,
  Classes,
  Controls,
  Dialogs;

type
  TDriveType = (
    dtUnknown=0,
    dtNoRootDir=1,
    dtRemovable=2,
    dtFixed=3,
    dtRemote=4,
    dtCDRom=5,
    dtRamDisk=6
  );
  TDriveTypes = set of TDriveType;

  TSelectFolderDialog = class(TComponent)
  private
    fRootDir          : TFileName;
    fTitle            : TCaption;
    fOptions          : TSelectDirExtOpts;
    fDirectory        : TFileName;
    fValidDrives      : TDriveTypes;
    fTotalSpaceNeeded : int64;
    fFreeSpaceNeeded  : int64;
    fFreeSpace        : int64;
    fTotalSpace       : int64;
    fTryAgain         : boolean;
    fIsValidDirectory : boolean;
    fOnShow           : TNotifyEvent;
    fOnClose          : TNotifyEvent;
  published
    // prompt user for try again to select a valid folder
    property TryAgain         : boolean           read fTryAgain write fTryAgain default true;

    // standard selectDir options
    property Options          : TSelectDirExtOpts read fOptions  write fOptions;

    // dialog title
    property Title            : TCaption          read fTitle    write fTitle;

    // dialog root dir at start
    property RootDir          : TFileName         read fRootDir  write fRootDir;

    // free space requiered
    property FreeSpaceNeeded  : int64             read fFreeSpaceNeeded  write fFreeSpaceNeeded default 0;

    // total space requiered
    property TotalSpaceNeeded : int64             read fTotalSpaceNeeded write fTotalSpaceNeeded default 0;

    // drive type supported, harddisk, cdrom etc.
    property ValidDriveTypes  : TDriveTypes       read fValidDrives      write fValidDrives;

    property OnShow           : TNotifyEvent      read fOnShow   write fOnShow;
    property OnClose          : TNotifyEvent      read fOnClose  write fOnClose;
  public
    // user folder selection
    property Directory        : TFileName         read fDirectory;

    // true if folder is valid (location, drive type, freespace...)
    property IsValidDirectory : boolean           read fIsValidDirectory;

    // folder disk location real free space
    property FreeSpace        : int64             read fFreeSpace;

    // folder disk location real total space
    property TotalSpace       : int64             read fTotalSpace;

    // execute dialog
    function Execute: boolean;

    constructor Create(AOwner: TComponent); override;
  end;

var
  SelectFolderWarningMessage : string = 'Oops : Selection not a valid folder! Try again ?';

procedure Register;

implementation

procedure Register;
begin
  RegisterComponents('Dialogs', [TSelectFolderDialog]);
end;

{ TSelectDirectoryDialog }

constructor TSelectFolderDialog.Create(AOwner: TComponent);
begin
  inherited;
  fRootDir    := '';
  fTotalSpaceNeeded := 0;
  fTotalSpace := 0;
  fFreeSpace  := 0;
  fFreeSpaceNeeded  := 0;
  fTitle       := 'Select a directory';
  fValidDrives := [dtRemovable, dtFixed, dtRemote, dtCDRom, dtRamDisk];
  fOptions     := [sdNewUI];
  fTryAgain    := true;
  fIsValidDirectory := false;

end;

function TSelectFolderDialog.Execute: boolean;
var
  DriveType : Integer;
  TryOK : Boolean;
begin
  if Assigned(fOnShow) then
    fOnShow(Self);

  result := false;
  repeat
    DriveType         := DRIVE_UNKNOWN;
    fIsValidDirectory := false;
    TryOK             := false;
    if FileCtrl.SelectDirectory(string(fTitle), string(fRootDir), string(fDirectory), fOptions) then
    begin
      fIsValidDirectory := Length(Directory) > 2;
      if fIsValidDirectory then
      begin
        DriveType          := GetDriveType(PChar(Directory[1]+':\'));
        fIsValidDirectory  := TDriveType(DriveType) in fValidDrives;
        if fIsValidDirectory then
        begin
          GetDiskFreeSpaceEx(PChar(Directory), fFreeSpace, fTotalSpace, nil);
          fIsValidDirectory := (fFreeSpace >= fFreeSpaceNeeded) and (fTotalSpace >= fTotalSpaceNeeded);
        end;
      end;
    end;
    if (not fIsValidDirectory) and fTryAgain then
      TryOK := MessageDlg(SelectFolderWarningMessage, mtWarning, [mbOk, mbAbort], 0) = mrOK;
  until fIsValidDirectory or not TryOk;
  result := fIsValidDirectory;

  if Assigned(fOnClose) then
    fOnClose(Self);
end;


end.
